@@ -619,7 +619,7 @@ delete_selection (gpointer data)
 }
 static void show_candwords(HildonIMFcitxFKBPrivate *priv)
 {
-  gchar tmp[MAX_USER_INPUT + 1];
+  gchar tmp[MAX_WORDS_USER_INPUT * UTF8_MAX_LENGTH + 1];
   gchar *wd = tmp;
   tmp[0] = '\0';
   if(iPYInsertPoint){
@@ -660,6 +660,7 @@ text_button_clicked (GtkButton *button, gpointer user_data)
 
   text = gtk_button_get_label(button);
   if(chinese_mode){
+    if(iPYInsertPoint >= MAX_USER_INPUT) return;
     if(('a' <= *text && 'z' >= *text) || ',' == *text || '.' == *text){
       if(iPYInsertPoint == 0 && ',' == *text)
         text = "，";
@@ -1059,7 +1060,7 @@ populate_window (HildonIMFcitxFKB *self)
   priv->buttons[1][9] = gtk_button_new();
   gtk_button_set_focus_on_click(GTK_BUTTON(priv->buttons[1][9]),FALSE);
   gtk_widget_set_size_request(priv->buttons[1][9], button_width*10, BUTTON_HEIGHT);
-  g_signal_connect (priv->buttons[1][9], "clicked", G_CALLBACK(close_fkb), self);
+  g_signal_connect (priv->buttons[1][9], "released", G_CALLBACK(close_fkb), self);
   gtk_box_pack_start(GTK_BOX(priv->vbox), priv->buttons[1][9], FALSE, FALSE, 0);
 
   priv->text_view = gtk_text_view_new();
@@ -1107,7 +1108,7 @@ populate_window (HildonIMFcitxFKB *self)
         priv->buttons[3][i] = gtk_button_new();
         gtk_button_set_focus_on_click(GTK_BUTTON(priv->buttons[3][i]),FALSE);
         gtk_widget_set_size_request(priv->buttons[3][i], -1, BUTTON_HEIGHT+fix_height);
-        g_signal_connect(G_OBJECT(priv->buttons[3][i]), "clicked", G_CALLBACK(text_button_clicked), self);
+        g_signal_connect(G_OBJECT(priv->buttons[3][i]), "released", G_CALLBACK(text_button_clicked), self);
         gtk_box_pack_start(GTK_BOX(priv->buttons[3][2]), GTK_WIDGET(priv->buttons[3][i]), TRUE, TRUE, 0);
   }
   gtk_box_pack_start(GTK_BOX(priv->vbox), priv->buttons[3][2], FALSE, FALSE, 0);
@@ -1146,7 +1147,7 @@ populate_window (HildonIMFcitxFKB *self)
       gtk_button_set_focus_on_click(GTK_BUTTON(priv->mode_button),FALSE);
       gtk_widget_set_size_request(priv->mode_button, button_width, BUTTON_HEIGHT+fix_height);
       gtk_box_pack_start(GTK_BOX(hbox), priv->mode_button, FALSE, FALSE, 0);
-      g_signal_connect(priv->mode_button, "clicked", G_CALLBACK(switch_button_clicked), self);
+      g_signal_connect(priv->mode_button, "released", G_CALLBACK(switch_button_clicked), self);
 
       /* Space button */
 
@@ -1186,7 +1187,7 @@ populate_window (HildonIMFcitxFKB *self)
                                                        -1));
       gtk_widget_set_size_request(priv->return_button, button_width*1.5, BUTTON_HEIGHT+fix_height);
       gtk_box_pack_start(GTK_BOX(hbox), priv->return_button, FALSE, FALSE, 0);
-      g_signal_connect(priv->return_button, "clicked", G_CALLBACK(return_button_clicked), self);
+      g_signal_connect(priv->return_button, "released", G_CALLBACK(return_button_clicked), self);
     }
     gtk_box_pack_start(GTK_BOX(priv->vbox), hbox, FALSE, FALSE, 0);
   }
